//--------------------------------------------------------------------
//- OpenGL - Partikel Systeme
//- Tutorial 1 - ber den Wolken...
//--
//- @author Tobias Rick
//- @version 1.0
//- @date: 04.03.02
//--------------------------------------------------------------------

#include "main.h"

#include "partemitter.h"

bool  g_bFullScreen = true;						
HWND  g_hWnd;											
RECT  g_rRect;										
HDC   g_hDC;											
HGLRC g_hRC;											
HINSTANCE g_hInstance;						

GLuint	g_Texture[MAX_TEXTURES];		// Speicher fr Texturen
CTimer g_Timer;
CPartEmitter* g_pEmitter = NULL;
float g_fLastUpdate = 0;

float g_vPos[3] = {0,0,0};	// Cameraposition
float g_fRot = 0;						// Cameradrehung
bool g_bDir = true;					// Richtung der Camerabewegung

void BuildParticles();

void InitParticles()
{
	// neues Emitterobjekt das bis zu 500 Partikel verwalten kann
	g_pEmitter = new CPartEmitter(500);
	
	g_Timer.Init();

	g_Timer.Update();
	CPartEmitter::m_fCurTime = g_Timer.GetCurrentTime();
	CPartEmitter::m_fFrameTime = g_Timer.GetFrameTime();

	// Partikel zum Emitter hinzufgen
	BuildParticles();
}

void BuildParticles()
{
	for(int i = 0; i < 200; i++)
	{
		CParticle* p = new CParticle(g_pEmitter);	// neuer Partikel
		
		// Partikel entstehen um (-10,-10,-150) + Zufallsvektor 
		VECTOR vPos = {-10 + ((rand() % 80)-40),
										-10+((rand() % 80)-40),
										-150+((rand() % 60)-30)};
		
		// zufllige Geschwindigkeit
		VECTOR vVel= {( rand()%10)-5*0.001 ,( rand()%10)-5*0.001 ,( rand()%60)-15 *0.001};
				
		float clrCol[4] = { 0.7, 0.7, 0.7 ,1};	// helles Grau
		
		float fLife = (rand() % 10 )*0.1 + 5.0;	// mind. Lebensdauer: 5.0 + Zufall
		float fSize = 0.2f + ( (rand() % 10)  -5)*0.1; // variable Groesse, mind. 0.2
		
		p->Set(vPos,vVel,clrCol,fSize,fLife);	// Werte bergeben

		g_pEmitter->Add(p);	// Neues Partikelobjekt dem Emitter bergeben
	}
}



void Init(HWND hWnd)
{
	srand( (unsigned)time( NULL ) );	// Zufallszahlen generieren

	g_hWnd = hWnd;										
	GetClientRect(g_hWnd, &g_rRect);	
	
	InitOpenGL(g_rRect.right, g_rRect.bottom);	

	LoadGLTextures();	// Texturen laden

	InitParticles();	// Partikelsystem initialisieren

	// OpenGL Einstellungen
	glShadeModel(GL_SMOOTH);							
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);	
	glHint(GL_POINT_SMOOTH_HINT,GL_NICEST);				
	glClearDepth(1.0f);									
	glDisable(GL_DEPTH_TEST);							
	glClearColor(0.0f,0.0f,0.0f,1.0f);		// Schwarzer Hintergrund
		
	// Blending Einstellungen
	glEnable(GL_BLEND);									
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);		
	
	glEnable(GL_TEXTURE_2D);							

	glBindTexture(GL_TEXTURE_2D,g_Texture[0]);	// Schneetextur auswhlen
}



void KeyPressed()
{
	// keine Benutzereingaben mglich	
}

WPARAM MainLoop()
{
	MSG msg;

	while(1)											
	{													
		if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) 
        { 
			if(msg.message == WM_QUIT)					
				break;
            TranslateMessage(&msg);						
            DispatchMessage(&msg);						
        }
		else											
		{ 
			// Hauptschleife
			g_Timer.Update();
			CPartEmitter::m_fCurTime = g_Timer.GetCurrentTime();
			CPartEmitter::m_fFrameTime = g_Timer.GetFrameTime();

			KeyPressed();
			RenderScene();							
     } 
	}

	// Speicher freigeben
	Shutdown();				
	if (g_pEmitter)
	{
		delete g_pEmitter;
		g_pEmitter = NULL;
	}


	return(msg.wParam);									
}

void DrawBackground()
{
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	glDisable(GL_BLEND);
	glBindTexture(GL_TEXTURE_2D, g_Texture[1]);

	glBegin(GL_QUADS);
		glTexCoord2i(0,3);
		glVertex3f(-900,-15.5,-900);
		glTexCoord2i(3,3); 
		glVertex3f(900,-15.5,-900);
		glTexCoord2i(3,0); 
		glVertex3f(300,-15.5,500);
		glTexCoord2i(0,0); 
		glVertex3f(-300,-15.5,500);
	glEnd();
	
	glBegin(GL_QUADS);
		glTexCoord2i(0,1);
		glVertex3f(-900,-15.5,-900);
		glTexCoord2i(1,1); 
		glVertex3f(900,-15.5,-900);
		glTexCoord2i(1,0); 
		glVertex3f(300,150.5,500);
		glTexCoord2i(0,0); 
		glVertex3f(-300,150.5,500);
	glEnd();
	

	glBindTexture(GL_TEXTURE_2D, g_Texture[0]);
	glEnable(GL_BLEND);
}

void RenderScene() 
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	
	glLoadIdentity();									
	
	glPushMatrix();

	if(g_bDir)
		glRotatef(g_fRot, 0.0f, 0.0f, 1.0f);
	g_fRot += g_Timer.GetFrameTime()*10;

	glTranslatef(g_vPos[0],g_vPos[1],g_vPos[2]);
	

	if(g_vPos[2] > 0 && !g_bDir)
	{
		g_vPos[2] -= g_Timer.GetFrameTime()*20;
	} else if (!g_bDir)
		g_bDir = true;

	if(g_vPos[2] < 700 && g_bDir)
	{
		g_vPos[2] += g_Timer.GetFrameTime()*10;
	} else if(g_vPos[2] >= 700)
		g_bDir = false;


	if(g_Timer.GetCurrentTime() - g_fLastUpdate  > 1.0f)
	{
		BuildParticles();
		g_fLastUpdate = g_Timer.GetCurrentTime();
	}
	DrawBackground();	// Hintergrund zeichnen

	glPopMatrix();	

	g_pEmitter->Update();	// Partikel zeichnen

	SwapBuffers(g_hDC);									
}


LRESULT CALLBACK WinProc(HWND hWnd,UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	LONG    lRet = 0; 
	PAINTSTRUCT    ps;
  
	switch (uMsg)
	{ 
		case WM_SYSCOMMAND:          
			{
				switch (wParam)             
				{                
					case SC_SCREENSAVE:       // Bildschirmschoner deaktivieren
					return FALSE;             
				}          
			}			break; 

    case WM_SIZE:										
		if(!g_bFullScreen)								
		{
			ReSize(LOWORD(lParam),HIWORD(lParam));
			GetClientRect(hWnd, &g_rRect);				
		}
        break; 
 
	case WM_PAINT:										
		BeginPaint(hWnd, &ps);					
		EndPaint(hWnd, &ps);						
		break;


	case WM_KEYDOWN:									
		switch (wParam)									
		{
			case VK_ESCAPE:								
				PostQuitMessage(0);					
				break;
		}
		break;

    case WM_CLOSE:										
        PostQuitMessage(0);						
        break; 
     
    default:											
        lRet = DefWindowProc (hWnd, uMsg, wParam, lParam); 
        break; 
    } 
 
    return lRet;									
}
